@@ -10,16 +10,20 @@ class TestCoroutinePool(greentest.TestCase):
 
     def test_apply_async(self):
         done = Event()
+
         def some_work(x):
             done.set()
+
         pool = self.klass(2)
         pool.apply_async(some_work, ('x', ))
         done.wait()
 
     def test_apply(self):
         value = 'return value'
+
         def some_work():
             return value
+
         pool = self.klass(2)
         result = pool.apply(some_work)
         self.assertEqual(value, result)
@@ -27,6 +31,7 @@ class TestCoroutinePool(greentest.TestCase):
     def test_multiple_coros(self):
         evt = Event()
         results = []
+
         def producer():
             results.append('prod')
             evt.set()
@@ -44,10 +49,13 @@ class TestCoroutinePool(greentest.TestCase):
 
     def dont_test_timer_cancel(self):
         timer_fired = []
+
         def fire_timer():
             timer_fired.append(True)
+
         def some_work():
             gevent.timer(0, fire_timer)
+
         pool = self.klass(2)
         pool.apply(some_work)
         gevent.sleep(0)
@@ -55,6 +63,7 @@ class TestCoroutinePool(greentest.TestCase):
 
     def test_reentrant(self):
         pool = self.klass(1)
+
         def reenter():
             result = pool.apply(lambda a: a, ('reenter', ))
             self.assertEqual('reenter', result)
@@ -62,6 +71,7 @@ class TestCoroutinePool(greentest.TestCase):
         pool.apply(reenter)
 
         evt = Event()
+
         def reenter_async():
             pool.apply_async(lambda a: a, ('reenter', ))
             evt.set()
@@ -75,10 +85,6 @@ class TestCoroutinePool(greentest.TestCase):
         # any members
         import sys
         pool = self.klass(size=1)
-        def crash(*args, **kw):
-            raise RuntimeError("Whoa")
-        class FakeFile(object):
-            write = crash
 
         # we're going to do this by causing the traceback.print_exc in
         # safe_apply to raise an exception and thus exit _main_loop
@@ -106,6 +112,14 @@ class TestCoroutinePool(greentest.TestCase):
             pool.join()
 
 
+def crash(*args, **kw):
+    raise RuntimeError("Whoa")
+
+
+class FakeFile(object):
+    write = crash
+
+
 class PoolBasicTests(greentest.TestCase):
     klass = pool.Pool
 
@@ -113,9 +127,8 @@ class PoolBasicTests(greentest.TestCase):
         p = self.klass(size=2)
         self.assertEqual(p.free_count(), 2)
         r = []
-        def foo(a):
-            r.append(a)
-        first = p.spawn(foo, 1)
+
+        first = p.spawn(r.append, 1)
         self.assertEqual(p.free_count(), 1)
         first.get()
         self.assertEqual(r, [1])
@@ -124,27 +137,32 @@ class PoolBasicTests(greentest.TestCase):
 
         #Once the pool is exhausted, calling an execute forces a yield.
 
-        p.apply_async(foo, (2, ))
+        p.apply_async(r.append, (2, ))
         self.assertEqual(1, p.free_count())
         self.assertEqual(r, [1])
 
-        p.apply_async(foo, (3, ))
+        p.apply_async(r.append, (3, ))
         self.assertEqual(0, p.free_count())
         self.assertEqual(r, [1])
 
-        p.apply_async(foo, (4, ))
+        p.apply_async(r.append, (4, ))
         self.assertEqual(r, [1])
         gevent.sleep(0.01)
-        self.assertEqual(r, [1,2,3,4])
+        self.assertEqual(r, [1, 2, 3, 4])
 
     def test_execute(self):
         p = self.klass()
         result = p.apply(lambda a: ('foo', a), (1, ))
         self.assertEqual(result, ('foo', 1))
 
+    def test_init_zerosize(self):
+        self.switch_expected = False
+        self.assertRaises(ValueError, self.klass, 0)
+
 #
 # tests from standard library test/test_multiprocessing.py
 
+
 class TimingWrapper(object):
 
     def __init__(self, func):
@@ -161,28 +179,31 @@ class TimingWrapper(object):
 
 def sqr(x, wait=0.0):
     gevent.sleep(wait)
-    return x*x
+    return x * x
 
 TIMEOUT1, TIMEOUT2, TIMEOUT3 = 0.082, 0.035, 0.14
 
+
 class TestPool(greentest.TestCase):
+    __timeout__ = 5
     size = 1
 
     def setUp(self):
+        greentest.TestCase.setUp(self)
         self.pool = pool.Pool(self.size)
 
-    def tearDown(self):
+    def cleanup(self):
         self.pool.join()
 
     def test_apply(self):
         papply = self.pool.apply
         self.assertEqual(papply(sqr, (5,)), sqr(5))
-        self.assertEqual(papply(sqr, (), {'x':3}), sqr(x=3))
+        self.assertEqual(papply(sqr, (), {'x': 3}), sqr(x=3))
 
     def test_map(self):
         pmap = self.pool.map
         self.assertEqual(pmap(sqr, range(10)), map(sqr, range(10)))
-        self.assertEqual(pmap(sqr, range(100)),  map(sqr, range(100)))
+        self.assertEqual(pmap(sqr, range(100)), map(sqr, range(100)))
 
     def test_async(self):
         res = self.pool.apply_async(sqr, (7, TIMEOUT1,))
@@ -196,7 +217,7 @@ class TestPool(greentest.TestCase):
         get = TimingWrapper(res.get)
         self.assertEqual(get(), 49)
         self.assertAlmostEqual(get.elapsed, TIMEOUT1, 1)
-        gevent.sleep(0) # let's the callback run
+        gevent.sleep(0)  # let's the callback run
         assert result == [49], result
 
     def test_async_timeout(self):
@@ -204,6 +225,7 @@ class TestPool(greentest.TestCase):
         get = TimingWrapper(res.get)
         self.assertRaises(gevent.Timeout, get, timeout=TIMEOUT2)
         self.assertAlmostEqual(get.elapsed, TIMEOUT2, 1)
+        self.pool.join()
 
     def test_imap(self):
         it = self.pool.imap(sqr, range(10))
@@ -211,12 +233,12 @@ class TestPool(greentest.TestCase):
 
         it = self.pool.imap(sqr, range(10))
         for i in range(10):
-            self.assertEqual(it.next(), i*i)
+            self.assertEqual(it.next(), i * i)
         self.assertRaises(StopIteration, it.next)
 
         it = self.pool.imap(sqr, range(1000))
         for i in range(1000):
-            self.assertEqual(it.next(), i*i)
+            self.assertEqual(it.next(), i * i)
         self.assertRaises(StopIteration, it.next)
 
     def test_imap_unordered(self):
@@ -227,21 +249,39 @@ class TestPool(greentest.TestCase):
         self.assertEqual(sorted(it), map(sqr, range(1000)))
 
     def test_terminate(self):
-        result = self.pool.map_async(gevent.sleep, [0.1] * 1000)
+        result = self.pool.map_async(gevent.sleep, [0.1] * ((self.size or 10) * 2))
+        gevent.sleep(0.1)
         kill = TimingWrapper(self.pool.kill)
-        kill(block=True)
+        kill()
         assert kill.elapsed < 0.5, kill.elapsed
+        result.join()
+
+    def sleep(self, x):
+        gevent.sleep(float(x) / 10.)
+        return str(x)
+
+    def test_imap_unordered_sleep(self):
+        # testing that imap_unordered returns items in competion order
+        result = list(self.pool.imap_unordered(self.sleep, [10, 1, 2]))
+        if self.pool.size == 1:
+            expected = ['10', '1', '2']
+        else:
+            expected = ['1', '2', '10']
+        self.assertEqual(result, expected)
 
 
 class TestPool2(TestPool):
     size = 2
 
+
 class TestPool3(TestPool):
     size = 3
 
+
 class TestPool10(TestPool):
     size = 10
 
+
 class TestPoolUnlimit(TestPool):
     size = None
 
@@ -250,8 +290,22 @@ class TestJoinSleep(greentest.GenericWaitTestCase):
 
     def wait(self, timeout):
         p = pool.Pool()
-        p.spawn(gevent.sleep, 10)
-        p.join(timeout=timeout)
+        g = p.spawn(gevent.sleep, 10)
+        try:
+            p.join(timeout=timeout)
+        finally:
+            g.kill()
+
+
+class TestJoinSleep_raise_error(greentest.GenericWaitTestCase):
+
+    def wait(self, timeout):
+        p = pool.Pool()
+        g = p.spawn(gevent.sleep, 10)
+        try:
+            p.join(timeout=timeout, raise_error=True)
+        finally:
+            g.kill()
 
 
 class TestJoinEmpty(greentest.TestCase):
@@ -270,12 +324,11 @@ class TestSpawn(greentest.TestCase):
         self.assertEqual(len(p), 0)
         p.spawn(gevent.sleep, 0.1)
         self.assertEqual(len(p), 1)
-        p.spawn(gevent.sleep, 0.1) # this spawn blocks until the old one finishes
+        p.spawn(gevent.sleep, 0.1)  # this spawn blocks until the old one finishes
         self.assertEqual(len(p), 1)
         gevent.sleep(0.19)
         self.assertEqual(len(p), 0)
 
 
-if __name__=='__main__':
+if __name__ == '__main__':
     greentest.main()
-
